--- old/usr/src/cmd/grep/grep.c	2013-09-14 23:25:09.596745499 +0100
+++ new/usr/src/cmd/grep/grep.c	2013-09-14 23:25:09.602220417 +0100
@@ -34,6 +34,7 @@
 
 /*
  * Copyright 2013 Damian Bogel. All rights reserved.
+ * Copyright (c) 2013 Andrew Stormont.  All rights reserved.
  */
 
 /*
@@ -102,6 +103,7 @@
 static int	hflag;
 static int 	Hflag;
 static int	qflag;
+static int	oflag;
 static int	errflg;
 static int	nfile;
 static long long	tln;
@@ -130,7 +132,7 @@
 #endif
 	(void) textdomain(TEXT_DOMAIN);
 
-	while ((c = getopt(argc, argv, "hHqblcnRrsviyw")) != -1)
+	while ((c = getopt(argc, argv, "hHqblcnoRrsviyw")) != -1)
 		switch (c) {
 		/* based on options order h or H is set as in GNU grep */
 		case 'h':
@@ -154,6 +156,9 @@
 		case 'n':
 			nflag++;
 			break;
+		case 'o':
+			oflag++;
+			break;
 		case 'R':
 			Rflag++;
 			/* FALLTHROUGH */
@@ -182,7 +187,7 @@
 		}
 
 	if (errflg || (optind >= argc)) {
-		errmsg("Usage: grep [-c|-l|-q] [-r|-R] -hHbnsviw "
+		errmsg("Usage: grep [-c|-l|-q|-o] [-r|-R] -hHbnsviw "
 		    "pattern file . . .\n",
 		    (char *)NULL);
 		exit(2);
@@ -413,8 +418,17 @@
 			lbuf = ptr;
 
 		/* lflag only once */
-		if ((step(lbuf, expbuf) ^ vflag) && succeed(file) == 1)
-			break;
+		if (step(lbuf, expbuf) ^ vflag) {
+			if (oflag) {
+				/*
+				 * Only store the matching bits
+				 */
+				ptr = loc1;
+				ptrend = loc2;
+			}
+			if (succeed(file) == 1)
+				break;
+		}
 
 		if (!nlflag)
 			break;
--- old/usr/src/man/man1/grep.1	2013-09-14 23:25:09.778962356 +0100
+++ new/usr/src/man/man1/grep.1	2013-09-14 23:25:09.783610228 +0100
@@ -1,4 +1,5 @@
 '\" te
+.\" Copyright (c) 2013 Andrew Stormont.  All rights reserved.
 .\" Copyright 2012 Nexenta Systems, Inc. All rights reserved.
 .\" Copyright 1989 AT&T
 .\" Copyright (c) 2008, Sun Microsystems, Inc.  All Rights Reserved
@@ -16,27 +17,27 @@
 .SH SYNOPSIS
 .LP
 .nf
-\fB/usr/bin/grep\fR [\fB-c\fR | \fB-l\fR |\fB-q\fR] [\fB-r\fR | \fB-R\fR] [\fB-bHhinsvw\fR]
+\fB/usr/bin/grep\fR [\fB-c\fR | \fB-l\fR | \fB-q\fR | \fB-o\fR] [\fB-r\fR | \fB-R\fR] [\fB-bHhinsvw\fR]
     \fIlimited-regular-expression\fR [\fIfilename\fR]...
 .fi
 
 .LP
 .nf
-\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR] [\fB-r\fR | \fB-R\fR]
+\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR | \fB-o\fR] [\fB-r\fR | \fB-R\fR]
     [\fB-bHhinsvwx\fR] \fB-e\fR \fIpattern_list\fR... [\fB-f\fR \fIpattern_file\fR]...
     [\fIfile\fR]...
 .fi
 
 .LP
 .nf
-\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR] [\fB-r\fR | \fB-R\fR]
+\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR | \fB-o\fR] [\fB-r\fR | \fB-R\fR]
     [\fB-bHhinsvwx\fR] [\fB-e\fR \fIpattern_list\fR]... \fB-f\fR \fIpattern_file\fR...
     [\fIfile\fR]...
 .fi
 
 .LP
 .nf
-\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR] [\fB-r\fR | \fB-R\fR]
+\fB/usr/xpg4/bin/grep\fR [\fB-E\fR | \fB-F\fR] [\fB-c\fR | \fB-l\fR | \fB-q\fR | \fB-o\fR] [\fB-r\fR | \fB-R\fR]
     [\fB-bHhinsvwx\fR] \fIpattern\fR [\fIfile\fR]...
 .fi
 
@@ -146,6 +147,15 @@
 .sp
 .ne 2
 .na
+\fB\fB-o\fR\fR
+.ad
+.RS 6n
+Print only the matching part of the line.
+.RE
+
+.sp
+.ne 2
+.na
 \fB\fB-r\fR\fR
 .ad
 .RS 6n
